@@ -7,23 +7,23 @@ use AI::MXNet::Function::Parameters;
 
 =head1 NAME
 
-AI::MXNet:Image - Read invidual image files and perform augmentations.
+    AI::MXNet:Image - Read invidual image files and perform augmentations.
 =cut
 
 =head2 imdecode
 
-Decode an image from string. Requires OpenCV to work.
+    Decode an image from string. Requires OpenCV to work.
 
-Parameters
-----------
-$buf : str, array ref, pdl, ndarray
-    Binary image data.
-:$flag : int
-    0 for grayscale. 1 for colored.
-:$to_rgb : int
-    0 for BGR format (OpenCV default). 1 for RGB format (MXNet default).
-:$out : NDArray
-    Output buffer. Do not specify for automatic allocation.
+    Parameters
+    ----------
+    $buf : str, array ref, pdl, ndarray
+        Binary image data.
+    :$flag : int
+        0 for grayscale. 1 for colored.
+    :$to_rgb : int
+        0 for BGR format (OpenCV default). 1 for RGB format (MXNet default).
+    :$out : NDArray
+        Output buffer. Do not specify for automatic allocation.
 =cut
 
 method imdecode(Str|PDL $buf, Int :$flag=1, Int :$to_rgb=1, Maybe[AI::MXNet::NDArray] :$out=)
@@ -46,14 +46,16 @@ method imdecode(Str|PDL $buf, Int :$flag=1, Int :$to_rgb=1, Maybe[AI::MXNet::NDA
 
 =head2 scale_down
 
-Scale down crop size if it's bigger than image size
+Scale down crop size if it's bigger than the image size.
 
-Parameters:
-Shape $src_size
-Shape $size
+    Parameters:
+    -----------
+    Shape $src_size
+    Shape $size
 
-Returns:
-($w, $h)
+    Returns:
+    --------
+    ($w, $h)
 =cut
 
 method scale_down(Shape $src_size, Shape $size)
@@ -73,15 +75,17 @@ method scale_down(Shape $src_size, Shape $size)
 
 =head2 resize_short
 
-Resize shorter edge to size
+    Resize shorter edge to the size.
 
-Parameters:
-AI::MXNet::NDArray $src
-Int                $size
-Int                $interp=2
+    Parameters:
+    -----------
+    AI::MXNet::NDArray $src
+    Int                $size
+    Int                $interp=2
 
-Returns:
-AI::MXNet::NDArray $resized_image
+    Returns:
+    --------
+    AI::MXNet::NDArray $resized_image
 =cut
 
 method resize_short(AI::MXNet::NDArray $src, Int $size, Int $interp=2)
@@ -101,19 +105,21 @@ method resize_short(AI::MXNet::NDArray $src, Int $size, Int $interp=2)
 
 =head2 fixed_crop
 
-Crop src at fixed location, and (optionally) resize it to size
+    Crop src at fixed location, and (optionally) resize it to the size.
 
-Parameters:
-AI::MXNet::NDArray $src
-Int                $x0
-Int                $y0
-Int                $w
-Int                $h
-Maybe[Shape]       $size=
-Int                $interp=2
+    Parameters:
+    -----------
+    AI::MXNet::NDArray $src
+    Int                $x0
+    Int                $y0
+    Int                $w
+    Int                $h
+    Maybe[Shape]       $size=
+    Int                $interp=2
 
-Returns:
-AI::MXNet::NDArray $cropped_image
+    Returns:
+    --------
+    AI::MXNet::NDArray $cropped_image
 =cut
 
 method fixed_crop(AI::MXNet::NDArray $src, Int $x0, Int $y0, Int $w, Int $h, Maybe[Shape] $size=, Int $interp=2)
@@ -128,15 +134,17 @@ method fixed_crop(AI::MXNet::NDArray $src, Int $x0, Int $y0, Int $w, Int $h, May
 
 =head2 random_crop
 
-Randomly crop src with size. Upsample result if src is smaller than size
+    Randomly crop src with size. Upsample result if src is smaller than the size.
 
-Parameters:
-AI::MXNet::NDArray $src
-Shape              $size=
-Int                $interp=2
+    Parameters:
+    -----------
+    AI::MXNet::NDArray $src
+    Shape              $size=
+    Int                $interp=2
 
-Returns:
-($cropped_image, [$x0, $y0, $new_w, $new_h])
+    Returns:
+    --------
+    ($cropped_image, [$x0, $y0, $new_w, $new_h])
 =cut
 
 method random_crop(AI::MXNet::NDArray $src, Shape $size, Int $interp=2)
@@ -153,15 +161,17 @@ method random_crop(AI::MXNet::NDArray $src, Shape $size, Int $interp=2)
 
 =head2 center_crop
 
-Randomly crop src with size around the center. Upsample result if src is smaller than size
+    Randomly crop src with size around the center. Upsample result if src is smaller than the size.
 
-Parameters:
-AI::MXNet::NDArray $src
-Shape              $size=
-Int                $interp=2
+    Parameters:
+    -----------
+    AI::MXNet::NDArray $src
+    Shape              $size=
+    Int                $interp=2
 
-Returns:
-($cropped_image, [$x0, $y0, $new_w, $new_h])
+    Returns:
+    --------
+    ($cropped_image, [$x0, $y0, $new_w, $new_h])
 =cut
 
 method center_crop(AI::MXNet::NDArray $src, Shape $size, Int $interp=2)
@@ -178,16 +188,18 @@ method center_crop(AI::MXNet::NDArray $src, Shape $size, Int $interp=2)
 
 =head2 color_normalize
 
-Normalize src with mean and std
+    Normalize src with mean and std.
 
-Parameter:
-AI::MXNet::NDArray $src
-Num|AI::MXNet::NDArray $mean
-Maybe[Num|AI::MXNet::NDArray] $std=
-Int $interp=2
+    Parameters:
+    -----------
+    AI::MXNet::NDArray $src
+    Num|AI::MXNet::NDArray $mean
+    Maybe[Num|AI::MXNet::NDArray] $std=
+    Int $interp=2
 
-Returns:
-AI::MXNet::NDArray $normalized_image
+    Returns:
+    --------
+    AI::MXNet::NDArray $normalized_image
 =cut
 
 method color_normalize(AI::MXNet::NDArray $src, Num|AI::MXNet::NDArray $mean, Maybe[Num|AI::MXNet::NDArray] $std=)
@@ -202,17 +214,19 @@ method color_normalize(AI::MXNet::NDArray $src, Num|AI::MXNet::NDArray $mean, Ma
 
 =head2 random_size_crop
 
-Randomly crop src with size. Randomize area and aspect ratio
+    Randomly crop src with size. Randomize area and aspect ratio.
 
-Parameters:
-AI::MXNet::NDArray $src
-Shape              $size
-Num                $min_area
-ArrayRef[Int]      [$from, $to] # $ratio
-Maybe[Int]         $interp=2
+    Parameters:
+    -----------
+    AI::MXNet::NDArray $src
+    Shape              $size
+    Num                $min_area
+    ArrayRef[Int]      [$from, $to] # $ratio
+    Maybe[Int]         $interp=2
 
-Returns:
-($cropped_image, [$x0, $y0, $new_w, $new_h])
+    Returns:
+    --------
+    ($cropped_image, [$x0, $y0, $new_w, $new_h])
 =cut
 
 method random_size_crop(AI::MXNet::NDArray $src, Shape $size, Num $min_area, ArrayRef[Num] $ratio, Maybe[Int] $interp=2)
@@ -249,15 +263,17 @@ method random_size_crop(AI::MXNet::NDArray $src, Shape $size, Num $min_area, Arr
 
 =head2 ResizeAug
 
-Makes "resize shorter edge to size augumenter" closure
+    Makes "resize shorter edge to size augumenter" closure.
 
-Parameters:
-Shape              $size
-Int                $interp=2
+    Parameters:
+    -----------
+    Shape              $size
+    Int                $interp=2
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [__PACKAGE__->resize_short($src, $size, $interp)]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [__PACKAGE__->resize_short($src, $size, $interp)]
 =cut
 
 method ResizeAug(Shape $size, Int $interp=2)
@@ -271,15 +287,17 @@ method ResizeAug(Shape $size, Int $interp=2)
 
 =head2 RandomCropAug
 
-Makes "random crop augumenter" closure
+    Makes "random crop augumenter" closure.
 
-Parameters:
-Shape              $size
-Int                $interp=2
+    Parameters:
+    -----------
+    Shape              $size
+    Int                $interp=2
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [(__PACKAGE__->random_crop($src, $size, $interp))[0]]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [(__PACKAGE__->random_crop($src, $size, $interp))[0]]
 =cut
 
 method RandomCropAug(Shape $size, Int $interp=2)
@@ -293,17 +311,18 @@ method RandomCropAug(Shape $size, Int $interp=2)
 
 =head2 RandomSizedCropAug
 
-Makes "random crop augumenter" closure
+    Makes "random crop augumenter" closure.
 
-Parameters:
-Shape              $size
-Num                $min_area
-ArrayRef[Num]      $ratio
-Int                $interp=2
+    Parameters:
+    -----------
+    Shape              $size
+    Num                $min_area
+    ArrayRef[Num]      $ratio
+    Int                $interp=2
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [(__PACKAGE__->random_size_crop($src, $size, $min_area, $ratio, $interp))[0]]
+    Returns:
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [(__PACKAGE__->random_size_crop($src, $size, $min_area, $ratio, $interp))[0]]
 =cut
 
 method RandomSizedCropAug(Shape $size, Num $min_area, ArrayRef[Num] $ratio, Int $interp=2)
@@ -317,15 +336,16 @@ method RandomSizedCropAug(Shape $size, Num $min_area, ArrayRef[Num] $ratio, Int
 
 =head2 CenterCropAug
 
-Makes "center crop augumenter" closure
+    Makes "center crop augumenter" closure.
 
-Parameters:
-Shape              $size
-Int                $interp=2
+    Parameters:
+    -----------
+    Shape              $size
+    Int                $interp=2
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [(__PACKAGE__->center_crop($src, $size, $interp))[0]]
+    Returns:
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [(__PACKAGE__->center_crop($src, $size, $interp))[0]]
 =cut
 
 method CenterCropAug(Shape $size, Int $interp=2)
@@ -339,14 +359,16 @@ method CenterCropAug(Shape $size, Int $interp=2)
 
 =head2 RandomOrderAug
 
-Makes "Apply list of augmenters in random order" closure
+    Makes "Apply list of augmenters in random order" closure.
 
-Parameters:
-ArrayRef[CodeRef]  $ts
+    Parameters:
+    -----------
+    ArrayRef[CodeRef]  $ts
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns ArrayRef[AI::MXNet::NDArray]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns ArrayRef[AI::MXNet::NDArray]
 =cut
 
 method RandomOrderAug(ArrayRef[CodeRef] $ts)
@@ -366,16 +388,18 @@ method RandomOrderAug(ArrayRef[CodeRef] $ts)
 
 =head2 RandomOrderAug
 
-Makes "Apply random brightness, contrast and saturation jitter in random order" closure
+    Makes "Apply random brightness, contrast and saturation jitter in random order" closure
 
-Parameters:
-Num $brightness
-Num $contrast
-Num $saturation
+    Parameters:
+    -----------
+    Num $brightness
+    Num $contrast
+    Num $saturation
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns ArrayRef[AI::MXNet::NDArray]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns ArrayRef[AI::MXNet::NDArray]
 =cut
 
 method ColorJitterAug(Num $brightness, Num $contrast, Num $saturation)
@@ -424,16 +448,18 @@ method ColorJitterAug(Num $brightness, Num $contrast, Num $saturation)
 
 =head2 LightingAug
 
-Makes "Add PCA based noise" closure
+    Makes "Add PCA based noise" closure.
 
-Parameters:
-Num $alphastd
-PDL $eigval
-PDL $eigvec
+    Parameters:
+    -----------
+    Num $alphastd
+    PDL $eigval
+    PDL $eigvec
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns ArrayRef[AI::MXNet::NDArray]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns ArrayRef[AI::MXNet::NDArray]
 =cut
 
 method LightingAug(Num $alphastd, PDL $eigval, PDL $eigvec)
@@ -450,15 +476,17 @@ method LightingAug(Num $alphastd, PDL $eigval, PDL $eigvec)
 
 =head2 ColorNormalizeAug
 
-Makes "Mean and std normalization" closure
+    Makes "Mean and std normalization" closure.
 
-Parameters:
-PDL $mean
-PDL $std
+    Parameters:
+    -----------
+    PDL $mean
+    PDL $std
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [__PACKAGE__->color_normalize($src, $mean, $std)]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [__PACKAGE__->color_normalize($src, $mean, $std)]
 =cut
 
 method ColorNormalizeAug(PDL $mean, PDL $std)
@@ -473,14 +501,16 @@ method ColorNormalizeAug(PDL $mean, PDL $std)
 
 =head2 HorizontalFlipAug
 
-Makes "Random horizontal flipping" closure
+    Makes "Random horizontal flipping" closure.
 
-Parameters:
-Num $p < 1
+    Parameters:
+    -----------
+    Num $p < 1
 
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [$p > rand ? AI::MXNet::NDArray->flip($src, axis=1>) : $src]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [$p > rand ? AI::MXNet::NDArray->flip($src, axis=1>) : $src]
 =cut
 
 method HorizontalFlipAug(Num $p)
@@ -493,14 +523,12 @@ method HorizontalFlipAug(Num $p)
 
 =head2 CastAug
 
-Makes "Cast to float32" closure
+    Makes "Cast to float32" closure.
 
-Parameters:
-Num $p < 1
-
-Returns:
-CodeRef that accepts AI::MXNet::NDArray $src as input
-and returns [$src->astype('float32')]
+    Returns:
+    --------
+    CodeRef that accepts AI::MXNet::NDArray $src as input
+    and returns [$src->astype('float32')]
 =cut
 
 method CastAug()
@@ -513,21 +541,22 @@ method CastAug()
 
 =head2 CreateAugmenter
 
-Create augumenter list
+    Create augumenter list
 
-Parameters:
-Shape          :$data_shape,
-Bool           :$resize=0,
-Bool           :$rand_crop=0,
-Bool           :$rand_resize=0,
-Bool           :$rand_mirror=0,
-Maybe[Num|PDL] :$mean=,
-Maybe[Num|PDL] :$std=,
-Num            :$brightness=0,
-Num            :$contrast=0,
-Num            :$saturation=0,
-Num            :$pca_noise=0,
-Int            :$inter_method=2
+    Parameters:
+    -----------
+    Shape          :$data_shape,
+    Bool           :$resize=0,
+    Bool           :$rand_crop=0,
+    Bool           :$rand_resize=0,
+    Bool           :$rand_mirror=0,
+    Maybe[Num|PDL] :$mean=,
+    Maybe[Num|PDL] :$std=,
+    Num            :$brightness=0,
+    Num            :$contrast=0,
+    Num            :$saturation=0,
+    Num            :$pca_noise=0,
+    Int            :$inter_method=2
 =cut
 
 method CreateAugmenter(
@@ -612,56 +641,60 @@ extends 'AI::MXNet::DataIter';
 
 =head1 NAME
 
-AI::MXNet::ImageIter - Image data iterator
+    AI::MXNet::ImageIter - Image data iterator.
 =cut
 
 =head1 DESCRIPTION
 
 
-Image data iterator with a large number of augumentation choices.
-Supports reading from both .rec files and raw image files with image list.
+    Image data iterator with a large number of augumentation choices.
+    Supports reading from both .rec files and raw image files with image list.
 
-To load from .rec files, please specify path_imgrec. Also specify path_imgidx
-to use data partition (for distributed training) or shuffling.
+    To load from .rec files, please specify path_imgrec. Also specify path_imgidx
+    to use data partition (for distributed training) or shuffling.
 
-To load from raw image files, specify path_imglist and path_root.
+    To load from raw image files, specify path_imglist and path_root.
 
-Parameters
-----------
-batch_size : Int
-    Number of examples per batch
-data_shape : Shape
-    Data shape in (channels, height, width).
-    For now, only RGB image with 3 channels is supported.
-label_width : Int
-    dimension of label
-path_imgrec : str
-    path to image record file (.rec).
-    Created with tools/im2rec.py or bin/im2rec
-path_imglist : str
-    path to image list (.lst)
-    Created with tools/im2rec.py or with custom script.
-    Format: index\t[one or more label separated by \t]\trelative_path_from_root
-imglist: array ref
-    a list of image with the label(s)
-    each item is a list [imagelabel: float or list of float, imgpath]
-path_root : str
-    Root folder of image files
-path_imgidx : str
-    Path to image index file. Needed for partition and shuffling when using .rec source.
-shuffle : bool
-    Whether to shuffle all images at the start of each iteration.
+    Parameters
+    ----------
+    batch_size : Int
+        Number of examples per batch
+    data_shape : Shape
+        Data shape in (channels, height, width).
+        For now, only RGB image with 3 channels is supported.
+    label_width : Int
+        dimension of label
+    path_imgrec : str
+        path to image record file (.rec).
+        Created with tools/im2rec.py or bin/im2rec
+    path_imglist : str
+        path to image list (.lst)
+        Created with tools/im2rec.py or with custom script.
+        Format: index\t[one or more label separated by \t]\trelative_path_from_root
+    imglist: array ref
+        a list of image with the label(s)
+        each item is a list [imagelabel: float or array ref of float, imgpath]
+    path_root : str
+        Root folder of image files
+    path_imgidx : str
+        Path to image index file. Needed for partition and shuffling when using .rec source.
+    shuffle : bool
+        Whether to shuffle all images at the start of each iteration.
     Can be slow for HDD.
-part_index : int
-    Partition index
-num_parts : int
-    Total number of partitions.
-kwargs : hash ref with any additional arguments for augmenters
+    part_index : int
+        Partition index
+    num_parts : int
+        Total number of partitions.
+    data_name='data' Str
+    label_name='softmax_label' Str
+    kwargs : hash ref with any additional arguments for augmenters
 =cut
 
 has 'batch_size'  => (is => 'ro', isa => 'Int',   required => 1);
 has 'data_shape'  => (is => 'ro', isa => 'Shape', required => 1);
 has 'label_width' => (is => 'ro', isa => 'Int',   default  => 1);
+has 'data_name'   => (is => 'ro', isa => 'Str',   default  => 'data');
+has 'label_name'  => (is => 'ro', isa => 'Str',   default  => 'softmax_label');
 has [qw/path_imgrec
         path_imglist
         path_root
@@ -746,7 +779,7 @@ sub BUILD
     assert(@{ $self->data_shape } == 3 and $self->data_shape->[0] == 3);
     $self->provide_data([
         AI::MXNet::DataDesc->new(
-            name  => 'data',
+            name  => $self->data_name,
             shape => [$self->batch_size, @{ $self->data_shape }]
         )
     ]);
@@ -754,7 +787,7 @@ sub BUILD
     {
         $self->provide_label([
             AI::MXNet::DataDesc->new(
-                name  => 'softmax_label',
+                name  => $self->label_name,
                 shape => [$self->batch_size, $self->label_width]
             )
         ]);
@@ -763,7 +796,7 @@ sub BUILD
     {
         $self->provide_label([
             AI::MXNet::DataDesc->new(
-                name  => 'softmax_label',
+                name  => $self->label_name,
                 shape => [$self->batch_size]
             )
         ]);
