@@ -2,7 +2,7 @@ package AI::MXNet::NDArray;
 
 =head1 NAME
 
-AI::MXNet::NDArray - Multidimensional tensor object of MXNet.
+    AI::MXNet::NDArray - Multidimensional tensor object of MXNet.
 =cut
 
 use strict;
@@ -205,12 +205,12 @@ method _sync_copyfrom(ArrayRef|PDL|PDL::Matrix $source_array)
 
 =head2 aspdl
 
-Returns a copied PDL array of current array.
+    Returns a copied PDL array of current array.
 
-Returns
--------
-array : PDL
-A copy of the array content.
+    Returns
+    -------
+    array : PDL
+        A copy of the array content.
 =cut
 
 method aspdl()
@@ -234,13 +234,14 @@ method aspdl()
 
 =head2 asmpdl
 
-Returns copied PDL::Matrix objectt of current array.
+    Returns copied PDL::Matrix objectt of current array.
 
-Requires caller to "use PDL::Matrix" in user space.
-Returns
--------
-array : PDL::Matrix
-A copy of array content.
+    Requires caller to "use PDL::Matrix" in user space.
+
+    Returns
+    -------
+    array : PDL::Matrix
+        A copy of array content.
 =cut
 
 method asmpdl()
@@ -264,14 +265,14 @@ method asmpdl()
 
 =head2 _slice
 
-Returns sliced NDArray that shares memory with the current one.
+    Returns sliced NDArray that shares memory with the current one.
 
-Parameters
-----------
-start : int
-    Starting index of slice.
-stop : int
-    Finishing index of slice.
+    Parameters
+    ----------
+    start : int
+        Starting index of slice.
+    stop : int
+        Finishing index of slice.
 =cut
 
 method  _slice (
@@ -292,12 +293,12 @@ method  _slice (
 
 =head2  _at
 
-Returns a sub NDArray that shares memory with current one.
+    Returns a sub NDArray that shares memory with current one.
 
-Parameters
-----------
-idx : int
-    index of the sub array.
+    Parameters
+    ----------
+    idx : int
+        index of the sub array.
 =cut
 
 
@@ -313,16 +314,25 @@ method _at(Index $idx)
 
 =head2 reshape
 
-Returns a reshaped NDArray that shares the memory with current one.
+    Returns a reshaped NDArray that shares the memory with current one.
+    One shape dimension can be -1. In this case, the value is inferred
+    from the length of the array and remaining dimensions.
 
-Parameters
-----------
-new_shape : Shape
-    new shape of NDArray
+    Parameters
+    ----------
+    new_shape : Shape
+        new shape of NDArray
 =cut
 
-method reshape(Shape $new_shape)
+method reshape(ArrayRef[Int] $new_shape)
 {
+    my $i = -1;
+    my @inferred = map { $i++; $_ == -1 ? ($i) : () } @$new_shape;
+    assert((@inferred <= 1), 'Only one dimension can be inferred.');
+    if(@inferred)
+    {
+        $new_shape->[$inferred[0]] = product(@{ $self->shape })/product(map { abs($_) } @{ $new_shape });
+    }
     my $handle = check_call(
                     AI::MXNetCAPI::NDArrayReshape(
                         $self->handle,
@@ -333,14 +343,60 @@ method reshape(Shape $new_shape)
     return __PACKAGE__->new(handle => $handle, writable => $self->writable);
 }
 
+=head2 ndim
 
-=head broadcast_to
+    Returns the number of dimensions of this array.
+=cut
 
-Broadcasting the current NDArray into the given shape. 
+method ndim()
+{
+    scalar(@{ $self->shape });
+}
 
-Parameters
----------
-Shape $shape : the shape to broadcast
+=head2 moveaxis
+
+    Moves the 'source' axis into the 'destination' position
+    while leaving the other axes in their original order
+
+    Parameters
+    ----------
+    source : int
+        Original position of the axes to move.
+    destination : int
+        Destination position for each of the original axes.
+
+    Returns
+    -------
+    result :NDArray
+    Array with moved axes.
+
+    Examples
+    --------
+    > $X = mx->nd->array([[1, 2, 3],
+                          [4, 5, 6]]);
+    > print Dumper($X->moveaxis(0, 1)->shape)
+    > [3, 2]
+=cut
+
+method moveaxis(Int $source, Int $dest)
+{
+    my @axes = 0..$self->ndim-1;
+    $source += @axes if $source < 0;
+    $dest += @axes if $dest < 0;
+    assert($source < @axes);
+    assert($dest < @axes);
+    my ($to_move) = splice(@axes, $source, 1);
+    splice(@axes, $dest, 0, $to_move);
+    return __PACKAGE__->transpose($self, \@axes);
+}
+
+=head2 broadcast_to
+
+    Broadcasting the current NDArray into the given shape. 
+
+    Parameters
+    ---------
+    Shape $shape : the shape to broadcast
 =cut
 
 method broadcast_to(Shape $shape)
@@ -362,7 +418,7 @@ method broadcast_to(Shape $shape)
     if(join(',',@$cur_shape) ne join(',',@{ $self->shape }))
     {
         return __PACKAGE__->SUPER::broadcast_to($self->reshape($cur_shape),{ shape => $shape });
-    }    
+    }
     else
     {
         return __PACKAGE__->SUPER::broadcast_to($self, { shape => $shape });
@@ -371,11 +427,11 @@ method broadcast_to(Shape $shape)
 
 =head2 wait_to_read
 
-Block until all pending write operations on the NDArray are finished.
+    Block until all pending write operations on the NDArray are finished.
 
-This function will return when all the pending writes to the current
-NDArray are finished. There can be pending reads going on when the
-function returns.
+    This function will return when all the pending writes to the current
+    NDArray are finished. There can be pending reads going on when the
+    function returns.
 =cut
 
 method wait_to_read()
@@ -385,11 +441,11 @@ method wait_to_read()
 
 =head2 shape
 
-Get the shape of current NDArray.
+    Get the shape of current NDArray.
 
-Returns
--------
-an array ref representing the shape of current ndarray
+    Returns
+    -------
+    an array ref representing the shape of current ndarray
 =cut
 
 method shape()
@@ -399,8 +455,7 @@ method shape()
 
 =head2 size
 
-Number of elements in the array.
-
+    Number of elements in the array.
 =cut
 
 method size(Shape|Undef $shape=)
@@ -413,11 +468,11 @@ method size(Shape|Undef $shape=)
 
 =head2 context
 
-The context of the NDArray.
+    The context of the NDArray.
 
-Returns
--------
-$context : AI::MXNet::Context
+    Returns
+    -------
+    $context : AI::MXNet::Context
 =cut
 
 method context()
@@ -433,13 +488,13 @@ method context()
 
 =head2 dtype
 
-The data type of current NDArray.
+    The data type of current NDArray.
 
-Returns
--------
-a data type string ('float32', 'float64', 'float16', 'uint8', 'int32') 
-representing the data type of the ndarray.
-'float32' is the default dtype for the ndarray class.
+    Returns
+    -------
+    a data type string ('float32', 'float64', 'float16', 'uint8', 'int32') 
+    representing the data type of the ndarray.
+    'float32' is the default dtype for the ndarray class.
 =cut
 
 method dtype()
@@ -454,20 +509,20 @@ method dtype()
 
 =head2 copyto
 
-Copy the content of current array to another entity.
+    Copy the content of current array to another entity.
 
-When another entity is the NDArray, the content is copied over.
-When another entity is AI::MXNet::Context, a new NDArray in the context
-will be created.
+    When another entity is the NDArray, the content is copied over.
+    When another entity is AI::MXNet::Context, a new NDArray in the context
+    will be created.
 
-Parameters
-----------
-other : NDArray or Context
-    Target NDArray or context we want to copy data to.
+    Parameters
+    ----------
+    other : NDArray or Context
+        Target NDArray or context we want to copy data to.
 
-Returns
--------
-dst : NDArray
+    Returns
+    -------
+    dst : NDArray
 =cut
 
 method copyto(AI::MXNet::Context|AI::MXNet::NDArray $other)
@@ -493,11 +548,11 @@ method copyto(AI::MXNet::Context|AI::MXNet::NDArray $other)
 
 =head2 copy
 
-Makes a copy of the current ndarray in the same context
+    Makes a copy of the current ndarray in the same context
 
-Returns
-------
-$copy : NDArray
+    Returns
+    ------
+    $copy : NDArray
 =cut
 
 method copy()
@@ -510,13 +565,13 @@ method copy()
 
 =head2 T
 
-Get transpose of the NDArray.
-Works only on 2-D matrices.
+    Get transpose of the NDArray.
+    Works only on 2-D matrices.
 =cut
 
 method T()
 {
-    if (@{$self->shape} != 2)
+    if (@{$self->shape} > 2)
     {
         confess('Only 2D matrix is allowed to be transposed');
     }
@@ -525,16 +580,16 @@ method T()
 
 =head2 astype
 
-Returns copied ndarray of current array with the specified type.
+    Returns copied ndarray of current array with the specified type.
 
-Parameters
-----------
-$dtype : Dtype
+    Parameters
+    ----------
+    $dtype : Dtype
 
-Returns
--------
-$array : ndarray
-A copy of the array content.
+    Returns
+    -------
+    $array : ndarray
+        A copy of the array content.
 =cut
 
 method astype(Dtype $dtype)
@@ -546,18 +601,18 @@ method astype(Dtype $dtype)
 
 =head2 as_in_context
 
-Returns an NDArray in the target context.
-If the array is already in that context, self is returned. Otherwise, a copy is
-made.
+    Returns an NDArray in the target context.
+    If the array is already in that context, self is returned. Otherwise, a copy is
+    made.
 
-Parameters
-----------
-context : AI::MXNet::Context
-The target context we want the return value to live in.
+    Parameters
+    ----------
+    context : AI::MXNet::Context
+        The target context we want the return value to live in.
 
-Returns
--------
-A copy or self as an NDArray in the target context.
+    Returns
+    -------
+        A copy or self as an NDArray in the target context.
 =cut
 
 method as_in_context(AI::MXNet::Context $context)
@@ -566,21 +621,21 @@ method as_in_context(AI::MXNet::Context $context)
     return $self->copyto($context);
 }
 
-=head onehot_encode
+=head2 onehot_encode
 
-One hot encoding indices into matrix out.
+    One hot encoding indices into matrix out.
 
-Parameters
-----------
-indices: NDArray
-    An NDArray containing indices of the categorical features.
+    Parameters
+    ----------
+    indices: NDArray
+        An NDArray containing indices of the categorical features.
 
-out: NDArray
-    The result of the encoding.
+    out: NDArray
+        The result of the encoding.
 
-Returns
--------
-$out: NDArray
+    Returns
+    -------
+        $out: NDArray
 =cut
 
 method onehot_encode(AI::MXNet::NDArray $indices, AI::MXNet::NDArray $out)
@@ -809,22 +864,25 @@ method true_divide(AI::MXNet::NDArray|Num $other, $reverse=)
     return $self->divide($other, $reverse);
 }
 
-=head2 empty(
+=head2 empty
 
-Creates an empty uninitialized NDArray, with the specified shape.
+    Creates an empty uninitialized NDArray, with the specified shape.
 
-Parameters
-----------
-shape : Shape
-    shape of the NDArray.
+    Parameters
+    ----------
+    $shape : Shape
+        shape of the NDArray.
 
-ctx : AI::MXNet::Context, optional
-The context of the NDArray, defaults to current default context.
+    :$ctx : AI::MXNet::Context, optional
+        The context of the NDArray, defaults to current default context.
 
-Returns
--------
-out: Array
-    The created NDArray.
+    :$dtype : Dtype, optional
+        The dtype of the NDArray, defaults to 'float32'.
+
+    Returns
+    -------
+    out: Array
+        The created NDArray.
 =cut
 
 method empty(Shape $shape, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_ctx, Dtype :$dtype='float32')
@@ -841,19 +899,23 @@ method empty(Shape $shape, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_
 
 =head2 zeros
 
-Creates a new NDArray filled with 0, with specified shape.
+    Creates a new NDArray filled with 0, with specified shape.
 
-Parameters
-----------
-shape : Shape
-    The shape of the NDArray.
-ctx : AI::MXNet::Context, optional.
-    The context of the NDArray, defaults to current default context.
+    Parameters
+    ----------
+    $shape : Shape
+        shape of the NDArray.
 
-Returns
--------
-out: Array
-    The created NDArray.
+    :$ctx : AI::MXNet::Context, optional
+        The context of the NDArray, defaults to current default context.
+
+    :$dtype : Dtype, optional
+        The dtype of the NDArray, defaults to 'float32'.
+
+    Returns
+    -------
+    out: Array
+        The created NDArray.
 =cut
 
 method zeros(Shape $shape, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_ctx, Dtype :$dtype='float32')
@@ -863,19 +925,23 @@ method zeros(Shape $shape, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_
 
 =head2 ones
 
-Creates a new NDArray filled with 1, with specified shape.
+    Creates a new NDArray filled with 1, with specified shape.
 
-Parameters
-----------
-shape : Shape
-    The shape of the NDArray.
-ctx : Context, optional.
-    The context of the NDArray, default to current default context.
+    Parameters
+    ----------
+    $shape : Shape
+        shape of the NDArray.
 
-Returns
--------
-out: Array
-    The created NDArray.
+    :$ctx : AI::MXNet::Context, optional
+        The context of the NDArray, defaults to current default context.
+
+    :$dtype : Dtype, optional
+        The dtype of the NDArray, defaults to 'float32'.
+
+    Returns
+    -------
+    out: Array
+        The created NDArray.
 =cut
 
 method ones(Shape $shape, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_ctx, Dtype :$dtype='float32')
@@ -885,21 +951,26 @@ method ones(Shape $shape, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_c
 
 =head2 full
 
-Creates a new NDArray filled with given value, with specified shape.
+    Creates a new NDArray filled with given value, with specified shape.
 
-Parameters
-----------
-shape : Shape
-    The shape of the NDArray.
-val : float or int
-    The value to be filled with.
-ctx : Context, optional.
-    The context of the NDArray, default to current default context.
+    Parameters
+    ----------
+    $shape : Shape
+        shape of the NDArray.
 
-Returns
--------
-out: NDArray
-    The created NDArray.
+    val : float or int
+        The value to be filled with.
+
+    :$ctx : AI::MXNet::Context, optional
+        The context of the NDArray, defaults to current default context.
+
+    :$dtype : Dtype, optional
+        The dtype of the NDArray, defaults to 'float32'.
+
+    Returns
+    -------
+    out: Array
+        The created NDArray.
 =cut
 
 method full(Shape $shape, Num $val, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_ctx, Dtype :$dtype='float32')
@@ -909,20 +980,23 @@ method full(Shape $shape, Num $val, AI::MXNet::Context :$ctx=AI::MXNet::Context-
 
 =head2 array
 
-Creates a new NDArray that is a copy of the source_array.
+    Creates a new NDArray that is a copy of the source_array.
 
-Parameters
-----------
-source_array : PDL, PDL::Matrix, Array ref in PDL::pdl format
+    Parameters
+    ----------
+    $source_array : PDL, PDL::Matrix, Array ref in PDL::pdl format
         Source data to create NDArray from.
 
-ctx : Context, optional
-The context of the NDArray, default to current default context.
+    :$ctx : AI::MXNet::Context, optional
+        The context of the NDArray, defaults to current default context.
 
-Returns
--------
-out: Array
-    The created NDArray.
+    :$dtype : Dtype, optional
+        The dtype of the NDArray, defaults to 'float32'.
+
+    Returns
+    -------
+    out: Array
+        The created NDArray.
 =cut
 
 method array(PDL|PDL::Matrix|ArrayRef $source_array, AI::MXNet::Context :$ctx=AI::MXNet::Context->current_ctx, Dtype :$dtype='float32')
@@ -945,22 +1019,22 @@ method array(PDL|PDL::Matrix|ArrayRef $source_array, AI::MXNet::Context :$ctx=AI
 
 =head2 concatenate
 
-Concatenates an array ref of NDArrays along the first dimension.
+    Concatenates an array ref of NDArrays along the first dimension.
 
-Parameters
-----------
-arrays :  array ref of NDArrays
-    Arrays to be concatenate. They must have identical shape except
-    for the first dimension. They also must have the same data type.
-axis : int
-    The axis along which to concatenate.
-always_copy : bool
-    Default 1. When not 1, if the arrays only contain one
-    NDArray, that element will be returned directly, avoid copying.
+    Parameters
+    ----------
+    $arrays :  array ref of NDArrays
+        Arrays to be concatenate. They must have identical shape except
+        for the first dimension. They also must have the same data type.
+    :$axis=0 : int
+        The axis along which to concatenate.
+    :$always_copy=1 : bool
+        Default is 1. When not 1, if the arrays only contain one
+        NDArray, that element will be returned directly, avoid copying.
 
-Returns
--------
-An NDArray in the same context as $arrays->[0]->context.
+    Returns
+    -------
+    An NDArray in the same context as $arrays->[0]->context.
 =cut
 
 method concatenate(ArrayRef[AI::MXNet::NDArray] $arrays, Index :$axis=0, :$always_copy=1)
@@ -1019,29 +1093,29 @@ method concatenate(ArrayRef[AI::MXNet::NDArray] $arrays, Index :$axis=0, :$alway
 
 =head2 arange
 
-Similar function in the MXNet ndarray as numpy.arange
-See Also https://docs.scipy.org/doc/numpy/reference/generated/numpy.arange.html.
+    Similar function in the MXNet ndarray as numpy.arange
+    See Also https://docs.scipy.org/doc/numpy/reference/generated/numpy.arange.html.
 
-Parameters
-----------
-start : number, optional
-    Start of interval. The interval includes this value. The default start value is 0.
-stop : number, optional
-    End of interval. The interval does not include this value.
-step : number, optional
-    Spacing between the values
-repeat : number, optional
-    The repeating time of all elements.
-    E.g repeat=3, the element a will be repeated three times --> a, a, a.
-ctx : Context, optional
-    The context of the NDArray, defaultw to current default context.
-dtype : data type, optional
-    The value type of the NDArray, defaults to float32
+    Parameters
+    ----------
+    :$start=0 : number, optional
+        Start of interval. The interval includes this value. The default start value is 0.
+    $stop= : number, optional
+        End of interval. The interval does not include this value.
+    :$step=1 : number, optional
+        Spacing between the values
+    :$repeat=1 : number, optional
+        The repeating time of all elements.
+        E.g repeat=3, the element a will be repeated three times --> a, a, a.
+    :$ctx : Context, optional
+        The context of the NDArray, defaultw to current default context.
+    :$dtype : data type, optional
+        The value type of the NDArray, defaults to float32
 
-Returns
--------
-out : NDArray
-    The created NDArray
+    Returns
+    -------
+    $out : NDArray
+        The created NDArray
 =cut
 
 method arange(Index :$start=0, Index :$stop=, Index :$step=1, Index :$repeat=1,
@@ -1059,26 +1133,26 @@ method arange(Index :$start=0, Index :$stop=, Index :$step=1, Index :$repeat=1,
 
 =head2 load
 
-Loads ndarray from a binary file.
+    Loads ndarrays from a binary file.
 
-You can also use Storable to do the job if you only work on perl.
-The advantage of load/save is the file is language agnostic.
-This means the file saved using save can be loaded by other language binding of mxnet.
-You also get the benefit being able to directly load/save from cloud storage(S3, HDFS)
+    You can also use Storable to do the job if you only work with Perl.
+    The advantage of load/save is the file is language agnostic.
+    This means the file saved using save can be loaded by other language binding of mxnet.
+    You also get the benefit being able to directly load/save from cloud storage(S3, HDFS)
 
-Parameters
-----------
-fname : str
-    The name of the file.Can be S3 or HDFS address (remember built with S3 support).
-    Example of fname:
+    Parameters
+    ----------
+    fname : str
+        The name of the file.Can be S3 or HDFS address (remember built with S3 support).
+        Example of fname:
 
-    - `s3://my-bucket/path/my-s3-ndarray`
-    - `hdfs://my-bucket/path/my-hdfs-ndarray`
-    - `/path-to/my-local-ndarray`
+        - `s3://my-bucket/path/my-s3-ndarray`
+        - `hdfs://my-bucket/path/my-hdfs-ndarray`
+        - `/path-to/my-local-ndarray`
 
-Returns
--------
-out : array ref of NDArrays or hash ref with NDArrays
+    Returns
+    -------
+    $out : array ref of NDArrays or hash ref with NDArrays
 =cut
 
 method load(Str $filename)
@@ -1101,25 +1175,25 @@ method load(Str $filename)
 
 =head2 save
 
-Save array ref of NDArray or hash of str->NDArray to a binary file.
+    Save array ref of NDArray or hash of str->NDArray to a binary file.
 
-You can also use Storable to do the job if you only work on perl.
-The advantage of load/save is the file is language agnostic.
-This means the file saved using save can be loaded by other language binding of mxnet.
-You also get the benefit being able to directly load/save from cloud storage(S3, HDFS)
+    You can also use Storable to do the job if you only work with Perl.
+    The advantage of load/save is the file is language agnostic.
+    This means the file saved using save can be loaded by other language binding of mxnet.
+    You also get the benefit being able to directly load/save from cloud storage(S3, HDFS)
 
-Parameters
-----------
-fname : str
-    The name of the file.Can be S3 or HDFS address (remember built with S3 support).
-    Example of fname:
+    Parameters
+    ----------
+    fname : str
+        The name of the file.Can be S3 or HDFS address (remember built with S3 support).
+        Example of fname:
 
-    - `s3://my-bucket/path/my-s3-ndarray`
-    - `hdfs://my-bucket/path/my-hdfs-ndarray`
-    - `/path-to/my-local-ndarray`
+        - `s3://my-bucket/path/my-s3-ndarray`
+        - `hdfs://my-bucket/path/my-hdfs-ndarray`
+        - `/path-to/my-local-ndarray`
 
-data : array ref of NDArrays hash ref of NDArrays
-    The data to be saved.
+    $data : array ref of NDArrays or hash ref of NDArrays
+        The data to be saved.
 =cut
 
 method save(Str $filename, ArrayRef[AI::MXNet::NDArray]|HashRef[AI::MXNet::NDArray] $data)
@@ -1150,26 +1224,26 @@ method save(Str $filename, ArrayRef[AI::MXNet::NDArray]|HashRef[AI::MXNet::NDArr
 
 =head2 imdecode
 
-Decode an image from string. Requires OpenCV to work.
+    Decode an image from string. Requires OpenCV to work.
 
-Parameters
-----------
-str_img : str
-    binary image data
-clip_rect : iterable of 4 int
-    clip decoded image to rectangle (x0, y0, x1, y1)
-out : NDArray
-    output buffer. can be 3 dimensional (c, h, w) or 4 dimensional (n, c, h, w)
-index : int
-    output decoded image to i-th slice of 4 dimensional buffer
-channels : int
-    number of channels to output. Decode to grey scale when channels = 1.
-mean : NDArray
-    subtract mean from decode image before outputting.
+    Parameters
+    ----------
+    $str_img : str
+        binary image data
+    :$clip_rect : iterable of 4 int
+        clip decoded image to rectangle (x0, y0, x1, y1)
+    :$out= : Maybe[NDArray]
+        output buffer. can be 3 dimensional (c, h, w) or 4 dimensional (n, c, h, w)
+    :$index : int
+        output decoded image to i-th slice of 4 dimensional buffer
+    :$channels=3 : int
+        number of channels to output. Decode to grey scale when channels = 1.
+    $mean= : Maybe[NDArray]
+        subtract mean from decode image before outputting.
 =cut
 
 method imdecode($str_img, ArrayRef[Int] :$clip_rect=[0, 0, 0, 0],
-                AI::MXNet::NDArray :$out=, Int :$index=0, Int :$channels=3, AI::MXNet::NDArray :$mean=)
+                Maybe[AI::MXNet::NDArray] :$out=, Int :$index=0, Int :$channels=3, Maybe[AI::MXNet::NDArray] :$mean=)
 {
     return __PACKAGE__->_imdecode(
         $mean//__PACKAGE__->_new_empty_handle(),
@@ -1183,13 +1257,13 @@ method imdecode($str_img, ArrayRef[Int] :$clip_rect=[0, 0, 0, 0],
 
 =head2 _new_empty_handle
 
-Returns a new empty handle.
+    Returns a new empty handle.
 
-Empty handle can be used to hold result
+    Empty handle can be used to hold result
 
-Returns
--------
-    a new empty ndarray handle
+    Returns
+    -------
+        a new empty ndarray handle
 =cut
 
 sub _new_empty_handle
@@ -1200,13 +1274,13 @@ sub _new_empty_handle
 
 =head2 _new_alloc_handle
 
-Returns a new handle with specified shape and context.
+    Returns a new handle with specified shape and context.
 
-Empty handle is only used to hold results
+    Empty handle is only used to hold results
 
-Returns
--------
-a new empty ndarray handle
+    Returns
+    -------
+    a new empty ndarray handle
 =cut
 
 func _new_alloc_handle($shape, $ctx, $delay_alloc, $dtype)
@@ -1224,8 +1298,8 @@ func _new_alloc_handle($shape, $ctx, $delay_alloc, $dtype)
 
 =head2 waitall
 
-Wait for all async operations to finish in MXNet.
-This function is used for benchmarks only.
+    Wait for all async operations to finish in MXNet.
+    This function is used for benchmarks only.
 =cut
 
 method waitall()
